@@ -14,8 +14,13 @@ import csv
 import signal
 import datetime
 import contextlib
+import os
+import sys
+import subprocess
 
 from awscli.compat import six
+from awscli.compat import get_binary_stdout
+from awscli.compat import get_popen_kwargs_for_pager_cmd
 
 
 def split_on_commas(value):
@@ -111,7 +116,7 @@ def _find_quote_char_in_part(part):
 
 def find_service_and_method_in_event_name(event_name):
     """
-    Grabs the service name and the operation name from an event name.
+    Grabs the service id and the operation name from an event name.
     This is making the assumption that the event name is in the form
     event.service.operation.
     """
@@ -141,3 +146,50 @@ def ignore_ctrl_c():
         yield
     finally:
         signal.signal(signal.SIGINT, original)
+
+
+def emit_top_level_args_parsed_event(session, args):
+    session.emit(
+        'top-level-args-parsed', parsed_args=args, session=session)
+
+
+def is_a_tty():
+    try:
+        return os.isatty(sys.stdout.fileno())
+    except Exception as e:
+        return False
+
+
+class OutputStreamFactory(object):
+    def __init__(self, popen=None):
+        self._popen = popen
+        if popen is None:
+            self._popen = subprocess.Popen
+
+    @contextlib.contextmanager
+    def get_pager_stream(self, preferred_pager=None):
+        popen_kwargs = self._get_process_pager_kwargs(preferred_pager)
+        try:
+            process = self._popen(**popen_kwargs)
+            yield process.stdin
+        except IOError:
+            # Ignore IOError since this can commonly be raised when a pager
+            # is closed abruptly and causes a broken pipe.
+            pass
+        finally:
+            process.communicate()
+
+    @contextlib.contextmanager
+    def get_stdout_stream(self):
+        yield get_binary_stdout()
+
+    def _get_process_pager_kwargs(self, pager_cmd):
+        kwargs = get_popen_kwargs_for_pager_cmd(pager_cmd)
+        kwargs['stdin'] = subprocess.PIPE
+        return kwargs
+
+
+def write_exception(ex, outfile):
+    outfile.write("\n")
+    outfile.write(six.text_type(ex))
+    outfile.write("\n")
